/* Example4
 * A few examples of how to use event selection predicates
 *
 * The Busy entity schedules lots of events for the Sleepy
 * entity, but Sleepy is always holding when they arrive, so
 * they get put on the deferred queue.
 */

import eduni.simjava.*;

class Sleepy extends Sim_entity {
  public Sleepy(String name) {
    super(name);
  }

  public void body() {
    Sim_event ev = new Sim_event();

    // Using Sim_select to get the first event off the deferred queue
    sim_hold(10.0);
    System.out.println(get_name()+": sim_waiting() returns "+sim_waiting());
    sim_select(Sim_system.SIM_ANY, ev);
    System.out.println(get_name()+": got waiting event, type: "+ev.get_tag());

    // Using the predicate Sim_type_p to get an event of a specific
    // type from the top of the deferred queue
    ev = new Sim_event();
    sim_hold(10.0);
    System.out.println(get_name()+": sim_waiting(2) returns "+sim_waiting(new Sim_type_p(2)));
    sim_select(new Sim_type_p(2), ev);
    //sim_select(Sim_system.SIM_ANY, ev);
    System.out.println(get_name()+": got waiting event, type: "+ev.get_tag());

    // Using sim_get_next to retrive events
    sim_select(Sim_system.SIM_ANY, ev);
    sim_get_next(ev);  // This one should get from the deferred queue
    System.out.println(get_name()+": sim_waiting() returns "+sim_waiting());
    sim_get_next(ev);  // This one should do a wait
  }
}

class Busy extends Sim_entity {
  public Busy(String name) {
    super(name);
  }

  public void body() {
   // One for sim_select
   sim_schedule(0, 1.0, 12);

   // Three for type selection predicate
   sim_schedule(0, 11.0, 1); 
   sim_schedule(0, 12.0, 2); 
   sim_schedule(0, 13.0, 3);

   // One for get_next doing a wait
   sim_schedule(0, 25.0, 13);
  }
}

class Example4 {
  public static void main(String args[]) {
    System.out.println("Main: initialising Sim_system");
    Sim_system.initialise();
    Sim_system.set_auto_trace(true);

    System.out.println("Main: creating entities");
    Sim_system.add(new Sleepy("Sleepy"));
    Sim_system.add(new Busy("Busy"));
    System.out.println("Main: about to call Sim_system.run()");
    Sim_system.run();
  }
}
